/**
 @file
 fftesi~ - SDK example of an object which accesses an MSP buffer~
 
 updated 3/22/09 ajm: new API
 
 @ingroup	examples	
 */

#include "ext.h"
#include "ext_obex.h"
#include "ext_common.h" // contains CLIP macro
#include "z_dsp.h"
#include "buffer.h"	// this defines our buffer's data structure and other goodies
#include "ext_atomic.h"
#include <fftw3.h>
#include <math.h>

void *fftesi_class;

typedef struct _fftesi
{
    t_pxobject l_obj;
    t_symbol *l_sym;
    t_buffer *l_buf;
	//t_atom fft[2048];
	//double fft_double[2049];
	int dim;
	fftw_complex *fft_result;
	fftw_plan plan;
	int i;
	float offset;
    
} t_fftesi;

//vogliamo mandare in output una lista
//t_symbol *ps_list;

//variabili per il calcolo della fft
double segnale[4096];


 // *fftesi_perform(t_int *w);
//void fftesi_dsp(t_fftesi *x, t_signal **sp);
void fftesi_set(t_fftesi *x, t_symbol *s);
void *fftesi_new(t_symbol *s, long chan);
void fftesi_bang(t_fftesi *x);
void fftesi_list(t_fftesi *x);
void fftesi_assist(t_fftesi *x, void *b, long m, long a, char *s);
void fftesi_dblclick(t_fftesi *x);
void copio_campioni(t_buffer *b);

t_symbol *ps_buffer;

int main(void)
{
	t_class *c;

	c = class_new("fftesi~", (method)fftesi_new, (method)dsp_free, (short)sizeof(t_fftesi), 0L, 
		A_SYM, A_DEFLONG, 0);
	//class_addmethod(c, (method)fftesi_dsp, "dsp", A_CANT, 0);
	//per fare una lista dovrebbe essere
	class_addmethod(c, (method)fftesi_set, "set", A_SYM, 0);
	//quando riceviamo un bang
	class_addmethod(c, (method)fftesi_bang, "bang",0);
	class_addmethod(c, (method)fftesi_list, "list", A_GIMME, 0);
	class_addmethod(c, (method)fftesi_assist, "assist", A_CANT, 0);
	class_addmethod(c, (method)fftesi_dblclick, "dblclick", A_CANT, 0);
	class_dspinit(c);
	class_register(CLASS_BOX, c);
	fftesi_class = c;
	
	ps_buffer = gensym("buffer~");
	//ps_list = gensym("list");

	return 0;
}

// here's where we set the buffer~ we're going to access
void fftesi_set(t_fftesi *x, t_symbol *s)
{
	t_buffer *b;
	
	if (s) {
		x->l_sym = s;
		if ((b = (t_buffer *)(s->s_thing)) && ob_sym(b) == ps_buffer) {
			x->l_buf = b;
		} else {
			object_error((t_object *)x, "no buffer~ %s", s->s_name);
			x->l_buf = 0;
		}
	} else {
		// this will reappear every time the dsp is restarted; do we really want it?
		object_error((t_object *)x, "no buffer~ object specified");
	}
}

void fftesi_bang(t_fftesi *x)
{
	post("bang received, provo a leggere il buffer");
	t_buffer *buf = x->l_buf;
	copio_campioni(buf);
}


void copio_campioni(t_buffer *b)
{
	float *tab;
	long i, chan;
	//double current_samp = 0.0;
	ATOMIC_INCREMENT((int32_t*)&b->b_inuse);
	post("speriamo bene");
	i = 1; chan = 1;
	if (!b->b_valid) {
		ATOMIC_DECREMENT(&b->b_inuse);
		return;
	}
	/*
	//copio il valore corrente in un array di dimensione 4096
	tab = b->b_samples;
	chan = 0; //usiamo solo il canale left
	for(i=0; i<b->b_frames; i++)
	{
		segnale[i] = tab[i];
		post ("letto campioni");
	}
	 */
	ATOMIC_DECREMENT((int32_t*)&b->b_inuse);
	return;
}


void fftesi_list(t_fftesi *x)
{
	post("write_list");
	return;
}

/*
 void fftesi_dsp(t_fftesi *x, t_signal **sp)
{
    fftesi_set(x,x->l_sym);
    dsp_add(fftesi_perform, 4, x, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n);
}
*/


// this lets us double-click on fftesi~ to open up the buffer~ it references
void fftesi_dblclick(t_fftesi *x)
{
	t_buffer *b;
	
	if ((b = (t_buffer *)(x->l_sym->s_thing)) && ob_sym(b) == ps_buffer)
		mess0((t_object *)b,gensym("dblclick"));
}


//su quale in/out sta puntando il mouse è dato dalla coppia di variabili m (in/out) e a (quale in o out)
void fftesi_assist(t_fftesi *x, void *b, long m, long a, char *s)
{
	if (m == ASSIST_OUTLET)
		sprintf(s,"(list) list of Amplitudes at freq n*1,5");
	else sprintf(s,"(bang) Perform FFT");
			
}

void *fftesi_new(t_symbol *s, long chan)
{
	t_fftesi *x = object_alloc(fftesi_class);
	outlet_new((t_object *)x, "fft_result");
	
	x->l_sym = s;
	return (x);
}

